@@ -1,73 +1,28 @@
 #!/usr/bin/env python
 """
-Crystallographic library for polarized neutron diffraction.
+CrysPy is a crystallographic library for neutron data analysis. It allows to
+refine polarized neutron diffraction experiments performed with single crystals
+as well as with powder magnetic compounds. The program has been mainly
+developed for Rietveld analysis of polarized neutron powder diffraction data.
+The program can be also used as a MEM tool to reconstruct magnetization
+(or spin) density. Time-of-flight (TOF) neutron data analysis is also
+available.
 
-Content
--------
-    Section A: Base functions
-    =========================
-        1. Algebra
-        ~~~~~~~~~~
-            - calc_scalar_product_by_vectors
-            - calc_scalar_product_by_complex_vectors
-            - calc_modulus_sq_by_complex_vector
-        2. Crystallography base
-        ~~~~~~~~~~~~~~~~~~~~~~~
-            - calc_volume_uc_by_abc_cosines
-            - calc_volume_uc_by_abc_angles
-            - calc_inverse_d_by_hkl_abc_cosines
-            - calc_inverse_d_by_hkl_abc_angles
-            - calc_sthovl_by_hkl_abc_cosines
-            - calc_sthovl_by_hkl_abc_angles
-            - calc_phase_3d
-            - calc_moment_2d_by_susceptibility
-            - ortogonalize_matrix
-        3. Extinction
-        ~~~~~~~~~~~~~
-            - calc_extinction
-            - calc_extinction_2
-        4. Flip ratio
-        ~~~~~~~~~~~~~
-            - calc_f_plus_sq
-            - calc_f_minus_sq
-            - calc_flip_ratio
-       5. Matrices
-            - tri_linear_interpolation
+Features
+  - Analysis of the polarized neutron scattering on crystals by the library
+    CrysPy;
+  - Diffraction data refinement for single crystals or powder by RhoChi;
+  - MEM for reconstruction of magnetization density;
+  - Powder refinement of magnetic space groups (under development).
+  - TOF neutron data analysis (under development)
 
-    Section B: Parent classes
-    =========================
-        1. Internal classes (not for user)
-            - ItemN, LoopN, DataN, GlobalN
-    Section C: Item/Loop classes
-    ============================
-        1. Classes to describe crystal strucutres
-            - unit_cell: Cell, CellL
-        2. Classes to describe experiments
-    Section D: Functions operating with item, loop
-    ==============================================
-        1.
-
-    Section E: Data classes
-    =======================
-        1.
-
-    Section F: Functions operating with Data classes
-    ================================================
-        1.
-
-    Section G: Global classes
-    =========================
-        1.
-
-    Section H: Functions operating with Global classes
-    ==================================================
-        1. str_to_globaln, file_to_globaln, str_to_items
+For more information please see the site: https://ikibalin.github.io/cryspy/
 """
 __author__ = 'Iurii KIBALIN'
 __copyright__   = "Copyright 2021, "
 __credits__ = ["Iurii KIBALIN", "Andrew SAZONOV", "Arsen GOUKASSOV"]
 __license__ = "GPL"
-__version__ = "0.5.3"
+__version__ = "0.5.5"
 __maintainer__ = "Iurii KIBALIN"
 __email__ = "iurii.kibalin@cea.fr"
 __status__ = "Development"
